@@ -28,7 +28,7 @@
 
 #include <string.h>
 #include <unistd.h>
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 
 #include <gtk/gtk.h>
 #include <camel/camel-provider.h>
@@ -392,7 +392,6 @@ check_node (GtkTreeStore *ts, ExchangeMAPIFolder *folder, GtkTreeIter iter)
 {
 	GtkTreeModel *ts_model;
 	mapi_id_t fid;
-	gboolean status = FALSE;
 
 	ts_model = GTK_TREE_MODEL (ts);
 	
@@ -408,14 +407,15 @@ check_node (GtkTreeStore *ts, ExchangeMAPIFolder *folder, GtkTreeIter iter)
 	if (gtk_tree_model_iter_has_child (ts_model, &iter)) {
 		GtkTreeIter child;
 		gtk_tree_model_iter_children (ts_model, &child, &iter);
-		status = check_node (ts, folder, child);
+		if (check_node (ts, folder, child))
+		    return TRUE;
 	}
 
-	while (gtk_tree_model_iter_next (ts_model, &iter) && !status) {
-		status = check_node (ts, folder, iter);
+	if (gtk_tree_model_iter_next (ts_model, &iter)) {
+		return check_node (ts, folder, iter);
 	}
 
-	return status;
+	return FALSE;
 }
 
 static void
