@@ -34,7 +34,7 @@
 #include <camel/camel-session.h>
 #include <camel/camel-url.h>
 #include <camel/camel-sasl.h>
-#include <camel/camel-i18n.h>
+#include <glib/gi18n-lib.h>
 
 #include "camel-mapi-store.h"
 #include "camel-mapi-transport.h"
@@ -53,24 +53,6 @@ static CamelProviderConfEntry mapi_conf_entries[] = {
 	  N_("C_heck for new messages in all folders"), "1" },
 	{ CAMEL_PROVIDER_CONF_SECTION_END },
 
-/* 	/\* override the labels/defaults of the standard settings *\/ */
-/* 	{ CAMEL_PROVIDER_CONF_LABEL, "username", NULL, */
-/* 	  /\* i18n: the '_' should appear before the same letter it */
-/* 	     does in the evolution:mail-config.glade "User_name" */
-/* 	     translation (or not at all) *\/ */
-/* 	  N_("Windows User_name:") }, */
-
-	/* extra Exchange configuration settings */
-	{ CAMEL_PROVIDER_CONF_SECTION_START, "activedirectory", NULL,
-	  /* i18n: GAL is an Outlookism, AD is a Windowsism */
-	  N_("Global Address List / Active Directory") },
-	{ CAMEL_PROVIDER_CONF_ENTRY, "ad_server", NULL,
-	  /* i18n: "Global Catalog" is a Windowsism, but it's a
-	     technical term and may not have translations? */
-	  N_("_Global Catalog server name:") },
-	{ CAMEL_PROVIDER_CONF_CHECKSPIN, "ad_limit", NULL,
-	  N_("_Limit number of GAL responses: %s"), "y:1:500:10000" },
-	{ CAMEL_PROVIDER_CONF_SECTION_END },
 	{ CAMEL_PROVIDER_CONF_SECTION_START, "generals", NULL,
 	  N_("Options") },
 	{ CAMEL_PROVIDER_CONF_CHECKBOX, "sync_offline", NULL,
@@ -128,6 +110,7 @@ void
 camel_provider_module_init(void)
 {
 	mapi_provider.name = "Exchange MAPI";
+	mapi_provider.translation_domain = (gchar *) GETTEXT_PACKAGE;
 	mapi_provider.auto_detect = mapi_auto_detect_cb;
 	mapi_provider.authtypes = g_list_prepend (mapi_provider.authtypes, &camel_mapi_password_authtype);
 	mapi_provider.url_hash = mapi_url_hash;
@@ -135,6 +118,10 @@ camel_provider_module_init(void)
 	mapi_provider.license = "LGPL";
 	mapi_provider.object_types[CAMEL_PROVIDER_STORE] = camel_mapi_store_get_type();
 	mapi_provider.object_types[CAMEL_PROVIDER_TRANSPORT] = camel_mapi_transport_get_type();
+
+	bindtextdomain (GETTEXT_PACKAGE, EXCHANGE_MAPI_LOCALEDIR);
+	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+
 	camel_provider_register (&mapi_provider);
 }
 
